#ifndef DUAL_SIZING_H
#define DUAL_SIZING_H

#include "../misc/timer.h"
#include "sizing_vars.h"
#include "flow_solver.h"

class dual_sizing
{
public:
	dual_sizing();

	bool solve(sizing_vars *p_vars, size_t pre_solve_rounds = 0);

	void solve_path(sizing_vars *p_vars);

	double compute_cost() const;

protected:
	void set_multipliers(double w);
	void update_flow(double w);
	double compute_derivative() const;
	double update_direction(double s);
	size_t compute_LRS(double w, bool check_feasible = true);
	virtual void attach_vars(sizing_vars *p_vars);
	void get_mp_as_flow();
	bool should_finish(int round);

	virtual bool find_next_flow(int round) = 0;
	
	sizing_vars *vars_;
	flow::CS_solver cs_solver_;

	std::vector<double> flow_, df_, old_vars_, old_flows_;
	std::vector<long> dfi_;

	double before_LRS_, old_cost_, min_change_;
	size_t LRS_iters_, cost_imp_count_;

	os::timer_vec LRS_, DF_;

	double scale_base_;

	// primal feasible solutions
	bool feasible_found_;
	double feasible_cp_, feasible_cost_, feasible_target_;

}; // class dual_sizing

#endif // DUAL_SIZING_H
