#ifndef FLOW_SIZING_H
#define FLOW_SIZING_H

#include "sizing_vars.h"

class flow_sizing
{
public:
	flow_sizing();

	bool solve_dual(sizing_vars *p_vars, bool exact);
  bool solve_dual(sizing_vars *p_vars, int max_iter, bool exact);
	double compute_cost() const;

protected:
	void collect_delays_and_bounds(int D_scale, int f_scale, double f_min_pos);
	void set_multipliers(double w);
	void update_flow(double w);
	double compute_derivative() const;
	double compute_max_w() const;
	bool update_direction();
	bool find_next_flow();
	bool find_next_flow_inexact();
	size_t compute_LRS(double w);
	void attach_vars(sizing_vars *p_vars);
	
	sizing_vars *vars_;

	std::vector<double> flow_, df_, df_last_;
	std::vector<int> D_, f_bound_, dfi_;

	double step_;
	double epsi_;
	double alpha_;

	bool cg_;
	size_t cg_count_;

	double before_LRS_;
	size_t total_iters_;
	size_t LRS_count_;
	int cs_count_;
	bool feasible_;

}; // class flow_sizing

#endif // FLOW_SIZING_H
