#ifndef PMATRIX_H
#define PMATRIX_H

#include "posynomial.h"

namespace math
{
  class pmatrix 
  {
  private:
    int nn;
    int mm;
    posynomial **v;
    
  public:
    //ctor
    pmatrix();
    pmatrix(int n, int m); //Zero-based array
    pmatrix(int n, int m, posynomial &p);
    pmatrix(const pmatrix &rhs);
  
    pmatrix &operator=(const pmatrix &rhs); //assignment
    
    inline posynomial* operator[](const int i)	//subscripting: pointer to row i
      {
	if (i<0 || i>=nn) {
	  assert(false && "pmatrix subscript out of bounds");
	}
	return v[i];
      }
    
    inline const posynomial* operator[](const int i) const
      {
	if (i<0 || i>=nn) {
	  assert(false && "pmatrix subscript out of bounds");
	}
	return v[i];
      }
    
    inline int nrows() const {
      return nn;
    }
    inline int ncols() const {
      return mm;
    }
    bool insert(int i, int j, posynomial& p);
    void resize(int newn, int newm); //resize, contents destroyed
    void assign(int newn, int newm, const posynomial &a);
    void save(FILE *fp, misc::cstr_t var_prefix, misc::cstr_t format = "e") const;
    
    void save(FILE *fp) const;
    bool load(FILE *fp);
    
    ~pmatrix();
  }; //class pmatrix : posynomial matrix
  
  inline pmatrix operator+(const pmatrix& a, const pmatrix &b)
  {
    if ( a.nrows() != b.nrows() && a.ncols() != b.ncols() )
      assert(false && "Matrix dimensions do not match");

    pmatrix temp(a.nrows(), a.ncols());
    posynomial p1, p2;
    for(int i = 0; i < a.nrows(); i++)
      for(int j = 0; j < a.ncols(); j++) {
	p1 = a[i][j];
	p2 = b[i][j];
	p1 += p2; 
	temp.insert(i,j,p1);
      }
    return temp;
  }

  inline pmatrix operator*(const pmatrix& a, const pmatrix &b)
  {
    if ( a.ncols() != b.nrows() )
      assert(false && "Matrix dimension incompatible for multiplication");

    int nn = a.nrows();
    int mm = b.ncols();
    pmatrix temp(nn,mm);
    
    for(int i = 0; i < nn; i++)
      for(int j = 0; j < mm; j++)
	{
	  posynomial p1, p2, p3;
	  for(int k = 0; k < a.ncols(); k++) {
	    p2 = a[i][k];
	    p3 = b[k][j];
	    p2 *= p3;
	    p1 += p2;
	  }
	  temp.insert(i,j,p1);
	}
    return temp;
  }
}

#endif
