#ifndef POSYNOMIAL_H
#define POSYNOMIAL_H

namespace math
{
  
class posynomial
{
public:
  struct product_term
  {
    size_t var_name;
    int pow_coef;
    
    product_term(size_t vn = 0, int pc = 1);

    bool operator==(const product_term &pt) {
      if ( var_name == pt.var_name ) return true;
      else return false;
    }
  }; // struct product_term
  
  typedef std::vector<product_term> product_terms;
  typedef std::vector<product_term>::iterator product_terms_iter;
  typedef std::vector<product_term>::const_iterator product_terms_citer;
  
  struct sum_term
  {
    product_terms prod_terms;
    double sum_coef;
    
    sum_term(double sc = 1);
    
    //Constructor to copy 
    sum_term(const sum_term &st){
      sum_coef = st.sum_coef;
      size_t n = st.prod_terms.size();
      for (size_t i = 0; i < n; i++) {
	size_t var_name = st.prod_terms[i].var_name;
	size_t pow_coef = st.prod_terms[i].pow_coef;
	add_term(var_name, pow_coef);
      }
    }
    
    sum_term(double coeff, product_terms pts){
      sum_coef = coeff;
      size_t n = pts.size();
      for (size_t i = 0; i < n; i++) {
	size_t var_name = pts[i].var_name;
	size_t pow_coef = pts[i].pow_coef;
	add_term(var_name,pow_coef);
      }
    }
    
    //Operator=
    sum_term & operator=(const sum_term &st) {
      sum_coef = st.sum_coef;
      prod_terms.clear();
      size_t n = st.prod_terms.size();
      for (size_t i = 0; i < n; i++) {
	size_t var_name = st.prod_terms[i].var_name;
	size_t pow_coef = st.prod_terms[i].pow_coef;
	add_term(var_name, pow_coef);
      }
      return *this;
    }
      
    //This function checks if the sum term has same product
    //term to which it is compared
      bool operator==(const sum_term &st) {
	if ( prod_terms.size() != st.prod_terms.size() )
	  return false;
	size_t num_match = 0;
	for(size_t i = 0; i < prod_terms.size(); i++) {
	  size_t var_name = prod_terms[i].var_name;
	  int pow_coef = prod_terms[i].pow_coef;
	  
	  for(size_t j = 0; j < st.prod_terms.size(); j++)
	    if ( var_name == st.prod_terms[j].var_name &&
		 pow_coef == st.prod_terms[j].pow_coef ) {
	      num_match++;
	      break;
	    }
	}
	if ( num_match == prod_terms.size() ) return true;
	else return false;
      }
    
    //This function gets the coefficient
    double get_coefficient() const {
      return sum_coef;
    }

    //This function updates the coefficient
    void set_coefficient(double coef) {
      sum_coef = coef;
    }
    
    double evaluate(const double values[]) const;
    
    std::pair<double, int> partial_evaluate(const double values[],
					    size_t which_var) const;
    std::string to_string(misc::cstr_t var_prefix,
			  misc::cstr_t format) const;
    std::string to_string(const misc::cstr_t var_names[],
			  misc::cstr_t format) const;

    void add_term(size_t var_name, int pow_coef);
    
    void save(FILE *fp) const;
    bool load(FILE *fp);    
  }; // struct sum_term
  
  typedef std::list<sum_term> sum_terms;
  typedef sum_terms::iterator sum_term_iter;
  typedef sum_terms::const_iterator sum_term_citer;
  typedef std::pair<sum_term_iter, sum_term_iter> sum_term_range;
  typedef std::pair<sum_term_citer, sum_term_citer> sum_term_crange;

  //constructor for a posynomial
  posynomial() {};
  
  posynomial(const posynomial &p) {
    terms_ = p.get_terms();
  }
  
  posynomial(posynomial &p) {
    terms_ = p.get_terms();
  }
  
  size_t num_terms() const;
  sum_terms &get_terms();
  const sum_terms &get_terms() const;
  sum_term_range get_term_range();
  sum_term_crange get_term_range() const;
  
  //get iterators
  
  sum_term_iter begin() {
    return terms_.begin();
  }
  
  sum_term_iter end() {
    return terms_.end();
  }
  
  //find whether the sum term exists
  sum_term_iter get_term(sum_term st) {
    return find(terms_.begin(), terms_.end(), st);
  }
  
  //Clear the sum terms
  void clear() {
    terms_.clear();
  }
  
  double evaluate(const double values[]) const;
  
  std::string to_string(misc::cstr_t var_prefix,
			misc::cstr_t format = "e") const;
  std::string to_string(const misc::cstr_t var_names[],
			misc::cstr_t format = "e") const;
  sum_term &add_term(double sum_coef = 1);
  
  void save(FILE *fp) const;
  bool load(FILE *fp);
  
  inline posynomial::sum_term
    merge_term(const posynomial::sum_term a, const posynomial::sum_term b)
    {
      double coeff1 = a.get_coefficient();
      double coeff2 = b.get_coefficient();
      
      posynomial::product_terms p1 = a.prod_terms;
      posynomial::product_terms p2 = b.prod_terms;
      
      posynomial::product_terms_iter p_iter1, p_iter2;
      posynomial::product_terms p3;
      
      for( p_iter1 = p1.begin(); p_iter1 != p1.end(); p_iter1++ ) {
	p3.push_back(*p_iter1);
      }
    
      for( p_iter2 = p2.begin(); p_iter2 != p2.end(); p_iter2++) {
	p_iter1 = find(p3.begin(), p3.end(), *p_iter2);
	if ( p_iter1 != p3.end() ) {
	  posynomial::product_term& p = *p_iter1;
	  p.pow_coef += (*p_iter2).pow_coef;
	} else {
	  p3.push_back(*p_iter2);
	}
      }
    
      double coeff = coeff1*coeff2;
      posynomial::sum_term st(coeff,p3);
      return st;
    }
  
  inline posynomial& operator+=(const posynomial& b)
  {
    posynomial::sum_term_crange cr_a = get_term_range();
    posynomial::sum_term_crange cr_b = b.get_term_range();
    
    posynomial ret;
    
    for(; cr_a.first != cr_a.second; cr_a.first++) {
      posynomial::sum_term &nst = ret.add_term();
      nst = *(cr_a.first);
    }
    clear();
    
    for(; cr_b.first != cr_b.second; cr_b.first++) {
      posynomial::sum_term st = *(cr_b.first);
      
      posynomial::sum_term_iter st_iter = ret.get_term(st);
      
      if (st_iter != ret.end() ) {
	posynomial::sum_term& st1 = *st_iter;
	double coeff = st.get_coefficient() + st1.get_coefficient();
	st1.set_coefficient(coeff);
      } else { 
	posynomial::sum_term& nst = ret.add_term();
	nst = st; 
      }
    }
    
    //if the coefficients are 0 then the sum_term has to be removed
    cr_a = ret.get_term_range();
    for(; cr_a.first != cr_a.second; ++cr_a.first) {
      posynomial::sum_term st = *(cr_a.first);
      if ( st.get_coefficient() == 0.0 ) continue;
      posynomial::sum_term& nst = add_term();
      nst = st;
    }
    ret.clear();
    
    return *this;
  }
  
  inline posynomial& operator*=(const posynomial& b)
  {
    posynomial::sum_term_crange cr_a = get_term_range();
    posynomial::sum_term_crange cr_b = b.get_term_range();
    
    posynomial ret;
    
    for(; cr_a.first != cr_a.second; cr_a.first++) 
      for(; cr_b.first != cr_b.second; cr_b.first++) {
	posynomial::sum_term st1 = *(cr_a.first);
	posynomial::sum_term st2 = *(cr_b.first);
	
	posynomial::sum_term st = merge_term(st1, st2);
	posynomial::sum_term_iter st_iter = ret.get_term(st);
	
	if (st_iter != ret.end() ) {
	  posynomial::sum_term& st1 = *st_iter;
	  double coeff = st.get_coefficient() + st1.get_coefficient();
	  st1.set_coefficient(coeff);
	} 
	else {
	  posynomial::sum_term& nst = ret.add_term();
	  nst = st;
	}
      }

    clear();
    
    cr_a = ret.get_term_range();
    for(; cr_a.first != cr_a.second; ++cr_a.first) {
      posynomial::sum_term st = *(cr_a.first);
      if ( st.get_coefficient() == 0.0 ) continue;
      posynomial::sum_term& nst = add_term();
      nst = st;
    }
    ret.clear();
    
    return *this;
  }
protected:
  sum_terms terms_;

}; // class posynomial
 
 inline posynomial
   operator+(posynomial& a, posynomial& b) {
   posynomial ret = a;
   ret += b;
   return ret;
 }
 
 inline posynomial
   operator*(posynomial& a, posynomial& b) {
   posynomial ret = a;
   ret *= b;
   return ret;
 }
}; // namespace math

#endif // POSYNOMIAL_H
