//@author : Debasish Das ddas@northwestern.edu
//sparse matrix representation of complex costs
//sparse matrix used from numerical recipes

#ifndef SPARSE_H
#define SPARSE_H

#include "nr3.h"

struct NRsparseMat
{
  Int nrows;
  Int ncols;
  Int nvals;
  VecInt col_ptr;
  VecInt row_ind;
  VecDoub val;
  
  NRsparseMat();
  NRsparseMat(Int m,Int n,Int nnvals);
  VecDoub ax(const VecDoub &x) const;
  VecDoub atx(const VecDoub &x) const;
  NRsparseMat transpose() const;
  bool generateDense(MatDoub& d);
  void save(FILE *fp) const;
  bool load(FILE *fp);
};

struct Linbcg {
	virtual void asolve(VecDoub_I &b, VecDoub_O &x, const Int itrnsp) = 0;
	virtual void atimes(VecDoub_I &x, VecDoub_O &r, const Int itrnsp) = 0;
	void solve(VecDoub_I &b, VecDoub_IO &x, const Int itol, const Doub tol,
		const Int itmax, Int &iter, Doub &err);
	Doub snrm(VecDoub_I &sx, const Int itol);
};

struct NRsparseLinbcg : Linbcg {
  NRsparseMat &mat;
  Int n;
  NRsparseLinbcg(NRsparseMat &matrix) : mat(matrix), n(mat.nrows) {}
  virtual void asolve(VecDoub_I &b, VecDoub_O &x, const Int itrnsp);
  virtual void atimes(VecDoub_I &x, VecDoub_O &r, const Int itrnsp);
};

#endif
