//There must be one element in each column of the sparse matrix
//Only then the column pointers will be valid 
#include "complex_cost.h"
#include "ludcmp.h"

int main(int argc, char * argv[])
{
  MatDoub a(5,5);
  
  for(int i = 0; i < a.nrows(); i++){
    for(int j = 0; j < a.ncols(); j++)
      a[i][j] = 0.0;
  }
  
  a[0][0] = 3.0;
  a[1][1] = 4.0;
  a[2][2] = 5.0;
  a[3][3] = 6.0;
  a[4][4] = 2.0;
  
  /*
  a[0][0] = 3.0;
  a[0][2] = 1.0;
  a[0][3] = 2.0;
  a[1][1] = 4.0;
  a[2][1] = 7.0;
  a[2][2] = 5.0;
  a[2][3] = 9.0;
  a[4][3] = 6.0;
  a[4][4] = 5.0;
  */
  
  for(int i = 0; i < a.nrows(); i++){
    for(int j = 0; j < a.ncols(); j++)
      printf("%g ", a[i][j]);
    printf("\n");
  }
  
  NRsparseMat conductance;
  conductance.nrows = a.nrows();
  conductance.ncols = a.ncols();
  int nnvals = 0;
  for(int i = 0; i < a.nrows(); i++) 
    for(int j = 0; j < a.ncols(); j++)
      if ( a[i][j] > 0.0)
	nnvals++;
  conductance.nvals = nnvals;
  conductance.col_ptr.resize(a.ncols()+1);
  conductance.row_ind.resize(nnvals);
  conductance.val.resize(nnvals);
  
  //Populate the sparse matrix
  size_t index = 0; 
  size_t col_ptr_index = 0;
  size_t last_col = 0;
  
  conductance.col_ptr[col_ptr_index] = last_col;
  col_ptr_index++;
  
  for(int j = 0; j < a.ncols(); j++)
    for(int i = 0; i < a.nrows(); i++)
      if ( a[i][j] > 0.0 ) {
	
	if ( last_col != j ) {
	  last_col = j;
	  conductance.col_ptr[col_ptr_index] = index;
	  col_ptr_index++;
	}
	
	conductance.val[index] = a[i][j];
	conductance.row_ind[index] = i;
	index++;
      }
  conductance.col_ptr[col_ptr_index] = index;
  
  printf("index k\n");
  for(size_t i = 0; i < conductance.val.size(); i++)
    printf("%lu ", i);
  printf("\n");
  
  printf("val[k]\n");
  for(size_t i = 0; i < conductance.val.size(); i++)
    printf("%g ",conductance.val[i]);
  printf("\n");
 
  printf("row_ind[k]\n");
  for(size_t i = 0; i < conductance.row_ind.size(); i++)
    printf("%d ", conductance.row_ind[i]);
  printf("\n");
  
  printf("column pointer\n");
  for(size_t i = 0; i < conductance.col_ptr.size(); i++)
    printf("%d ", conductance.col_ptr[i]);
  printf("\n");
  
#if 0
  printf("Solve by inverse computation using LU factorization\n");
  LUdcmp lu(a);
  MatDoub I(5,5);
  for(size_t i = 0; i < 5; i++)
    for(size_t j = 0; j < 5; j++) {
      if ( i == j ) I[i][j] = 1;
      else I[i][j] = 0;
    }
  MatDoub inv_a(5,5);
  lu.solve(I,inv_a);
  
  VecDoub b(5);
  b[0] = 1, b[1] = 1, b[2] = 1, b[3] = 1, b[4] = 1;
  
  VecDoub x(5);
  for(size_t i = 0; i < 5; i++)
    x[i] = 0;
  
  for(size_t i = 0; i < 5; i++)
    for(size_t j = 0; j < 5; j++)
      x[i] += inv_a[i][j]*b[j];
  
  for(size_t i = 0; i < 5; i++)
    printf("%g\n", x[i]);
#endif

#if 1
  printf("Solve using conjugate gradient method utilizing sparsity of matrix\n");
  VecDoub b(5);
  VecDoub x(5);
  for(size_t i = 0; i < 5; i++)
    b[i] = 1;
  NRsparseLinbcg lbcg(conductance);
  lbcg.asolve(b,x,0);
  for(size_t i = 0; i < 5; i++)
    printf("%g\n", x[i]);
#endif

}
