/*
 * Copyright (c) 2003-2005 Northwestern University.
 *                         NuCAD Group.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * graph2_v.cpp - Implementation of vector based graph.
 *
 * Authors: Jia Wang, jwa112@ece.northwestern.edu
 *
 */

#include "graph2_v.h"

namespace graph2
{

graph_v::graph_v(size_t n_V, const size_t edges[], size_t n_E)
{
	assign(n_V, edges, n_E);
}

void graph_v::assign(size_t n_V, const size_t edges[], size_t n_E)
{
	num_vertices_ = n_V;
	num_edges_ = n_E;

	edges_ = edges;

	inout_base_.resize(num_vertices()*2+2);
	inout_edges_.resize(num_edges()*2);

	// count vertices' degree
	std::vector<size_t> fanin_count(num_vertices(), 0), fanout_count(num_vertices(), 0);
	for (edge_descriptor e(0); valid_edge(e); ++e)
		++fanin_count[target(e).id()], ++fanout_count[source(e).id()];

	// make adjacent index
	inbase(0) = 0;
	outbase(0) = num_edges();
	inbase(num_vertices()) = num_edges();
	outbase(num_vertices()) = 2*num_edges();
	for (size_t j = 1; j < num_vertices(); ++j)
	{
		inbase(j) = inbase(j-1)+fanin_count[j-1];
		outbase(j) = outbase(j-1)+fanout_count[j-1];
	}

	// fill adjacent list
	for (edge_descriptor e(0); valid_edge(e); ++e)
	{
		size_t from = source(e).id(), to = target(e).id(), oi;

		// source's fanout
		oi = outbase(from+1)-fanout_count[from];
		inout_edges_[oi] = e;
		--fanout_count[from];

		// target's fanin
		oi = inbase(to+1)-fanin_count[to];
		inout_edges_[oi] = e;
		--fanin_count[to];
	}
}

} // namespace graph2
